/* max permitted O-O distance */
#define OO 4

/* H-bond limit */
#define HO 2.5

static void nearestdr(vector dr)
{
  int k;
  loop (k,0,3) {
    if (dr[k]>box.Lh[k]) dr[k]-=box.L[k];
    else if (dr[k]<-box.Lh[k]) dr[k]+=box.L[k]; }
}

void bjerrum_topology(ToIntPtr A) /************************ bjerrum_topology */
{
  int n,n1,n2,ns,i;
  molecule_t *m1;
  siteinfo_t *si1;
  vector *r1,*r2,dr;
  int nHB1,nHB2,nL,nL1,nL2,nD,nD1,nD2;
  int D1=-1,D2=-1,L1=-1,L2=-1; /* as read from the .oo file */

  static double maxm=0,minm=9e9;
  static int iO,iH1,iH2;
  static int (*hb)[2];

  if (maxm==0) {
    FILE *oo;
    char line[64];
    int n=0,a,b;

    allocarrayzero(hb,No.N*2);

    /* O,H sites */
    m1=molec;
    ns=m1->ns;
    si1=spec[m1->sp]->si;
    loop (i,0,ns) {
      if (si1[i].mass>maxm) {
        maxm=si1[i].mass; iO=i; }
      if (si1[i].mass>0 && si1[i].mass<minm) {
        minm=si1[i].mass*1.0001;
        iH1=iH2,iH2=i; } }
    prt("molecule 0 sites: O=%d H=%d,%d",iO,iH1,iH2);

    oo=fopen(Fn("oo"),"rt");
    if (!oo) ERROR(("bjerrum: neighbor list (H-bonded water-water pairs) %s missing",lastFn))

    while (fgets(line,64,oo)) {
      if (n>=2*No.N) ERROR(("%s: too many lines",lastFn))
      sscanf(line,"%d%d",&a,&b);
      if (a<0 || a>=No.N || b<0 || b>=No.N) ERROR(("%s: water molecule numbers out of range"))

      hb[n][0]=a;
      hb[n][1]=b;
      if (strchr(line,'D')) D1=a,D2=b;
      if (strchr(line,'L')) L1=a,L2=b;
      n++; }

    if (n<2*No.N) ERROR(("%s: too few lines (called because bj.mode&1)",lastFn))
    prt("%s read: D-defect: D1=%d D2=%d, L-defect: L1=%d L2=%d",
        lastFn,D1,D2,L1,L2); }

  /* finding a defect - topology way */
  nL=nD=0;
  prt("bjerrum: t=%f",t);
  loop (n,0,2*No.N) {
    n1=hb[n][0];
    n2=hb[n][1];
    m1=molec+n1;
    si1=spec[m1->sp]->si;
    r1=rof(m1,A->rp);
    r2=rof(molec+n2,A->rp);

    VVV(dr,=r1[iO],-r2[iO]) nearestdr(dr);

    if (SQR(dr)>Sqr(OO)) prt("O%d-O%d=%g exceeded the limit",n1,n2,sqrt(SQR(dr)));

    nHB1=0;
    VVV(dr,=r1[iO],-r2[iH1]) nearestdr(dr); if (SQR(dr)<Sqr(HO)) nHB1++;
    VVV(dr,=r1[iO],-r2[iH2]) nearestdr(dr); if (SQR(dr)<Sqr(HO)) nHB1++;

    nHB2=0;
    VVV(dr,=r2[iO],-r1[iH1]) nearestdr(dr); if (SQR(dr)<Sqr(HO)) nHB2++;
    VVV(dr,=r2[iO],-r1[iH2]) nearestdr(dr); if (SQR(dr)<Sqr(HO)) nHB2++;

    if (nHB1==0 && nHB2==0) {
      /* L-defect */
      nL++; nL1=n1; nL2=n2; }

    if (nHB1==1 && nHB2==1) {
      /* D-defect */
      nD++; nD1=n1; nD2=n2; } }

  prt_("nD=%d nD1=%d nD2=%d   ",nD,nD1,nD2);
  prt("nL=%d nL1=%d nL2=%d BJERRUM1",nL,nL1,nL2);
}

void bjerrum_gauss(ToIntPtr A,double sig) /******************* bjerrum_gauss */
{
  int n,i,k,idefect,nit;
  molecule_t *m;
  siteinfo_t *si;
  vector *r,dr,R0={0,0,0},R;
  double d,sigq,sumq,sumqm,rr;

  while (sig>1.5) {

    sigq=Sqr(sig)*2;

    loop (idefect,0,2) {
  
      sumqm=-9e9;
      d=pow(PROD(box.L),1./3);
      if (idefect) VV(R0,+=box.Lh) 
      nit=0;
  
      while (d>1e-8) {
        rndsph(dr);
        VVV(R,=R0,+d*dr)
        loop (k,0,3) {
          while (R[k]<0) R[k]+=box.L[k];
          while (R[k]>=box.L[k]) R[k]-=box.L[k]; }
    
        sumq=0;
    
        loop (n,0,No.N) {
          m=molec+n;
          si=spec[m->sp]->si;
          r=rof(m,A->rp);
          loop (i,0,m->ns) if (si[i].charge) {
            VVV(dr,=R,-r[i]) nearestdr(dr);
            rr=SQR(dr);
            sumq+=exp(-rr/sigq)*si[i].charge; } }
    
        if (idefect) sumq=-sumq;
    
        if (sumq>sumqm) {
          sumqm=sumq;
          VV(R0,=R)
          if (option('v')&4) prt("%3d [%10.6f %10.6f %10.6f]  %8.3g %9.6f BJERRUM2",
                                 nit,VARG(R),d,(idefect?-sumqm:sumqm)/electron);
          d*=1.1; }
        else 
          d*=0.98; 
        nit++; }
      sumqm/=electron;
      prt("%9.6f %9.6f [%10.6f %10.6f %10.6f] BJERRUM2",
          sig,idefect?-sumqm:sumqm,VARG(R0),nit); }
    sig*=0.97;
  }
}
